# @(c) Pedro Reis dos Santos, IST Ulisboa, 2026
"""
Introdução ao algoritmos e estruturas de dados

Algoritmos de pesquisa:
>>> help("iaed.search")

Algoritmos de ordenação:
>>> help("iaed.sorting")

Estruturas de dados: tabelas de dispersão (hashext, hashlin, hashdbl),
    árvores(btree, AVL), grafos(graph)
>>> help("iaed.hashext")
"""

__author__ = 'Pedro Reis dos Santos <reis.santos@tecnico.ulisboa.pt>'
__web__ = 'https://github.com/pedroreissantos/iaed.git'
__version__ = '1.0.3'
__all__ = ['search', 'sorting', 'hashext', 'hashlin', 'hashdbl',
        'btree', 'avl', 'graph', 'fib'] # visible modules within the package
